/*****************************************
*
* Description: Handle all window events 
*
*
******************************************/

/*
*  Function: eng_stop_event ()
*
*  Description: Stop any window event
*
*/
function eng_stop_event(parm_event, parm_type)
{
  trace ("eng_stop_event (parm_event, "+parm_type+")");
  
  if (!parm_event) {
    parm_event = window.event;
  }

  /* Netscape and mozila */
  if (parm_event.preventDefault) {
    parm_event.preventDefault();
  }
  if (parm_event.stopPropagation) {
    parm_event.stopPropagation();
  }

  /* Returning false also helps stop events */
  return false;
}

/*
*  Function: eng_handle_keydown ()
*
* Description: Store keycode for special keys
* First event fired. Special keys have the
* same keycode as regular ascii characters, so we
* have to use a combination of the keydown event
* and keypress event.
*
* NOTE: Stopping or not stopping the keydown event
*       doesn't seem to have any effect. More research
*       will have to be done to understand the impact of
*       this event on the final outcome of the key pressed.
*
*/
function eng_handle_keydown(parm_event)
{
  var key_code;
  var return_value = false;

  trace ("eng_handle_keydown (parm_event[keyCode: "+parm_event.keyCode+"])");

  /* Do not stop these keys until FF fixes some stuff */
  if (parm_event.keyCode == KEYCODE_DC || (parm_event.keyCode == KEYCODE_BACKSPACE && is_field_input (parm_event.target)) || parm_event.keyCode == KEYCODE_CAPSLOCK || (parm_event.keyCode == KEYCODE_RIGHT && is_field_input (parm_event.target)) || (parm_event.keyCode == KEYCODE_LEFT && is_field_input (parm_event.target)) || parm_event.keyCode == KEYCODE_SHIFT || (parm_event.keyCode == KEYCODE_IC && parm_event.shiftKey))
  {
    return true;
  }
  else
  {
    eng_stop_event (parm_event,'keydown');
    return false;
  }
}

/*
  *  Function: eng_handle_keypress ()
  *
  *  Description: Second event fired and sometimes
  *  the last. This is were we truely find out what
  *  key was pressed.
  *
*/
function eng_handle_keypress(parm_event)
{
  var key;
  var return_value = false;

  trace ("eng_handle_keypress (parm_event)");

  /* This a key fired from fire_keycode */
  if (parm_event.pass_key) {
    return true;
  }

  /* This gets the jscurses key */
  key = getch (parm_event);

  /* If the engine is not ready for data we must buffer
  the users keys */
  if (Sh_IO_state == SH_IO_READY)
  {
    if (key.keyCode == KEYCODE_DC || key.keyCode == KEYCODE_CAPSLOCK || (key.keyCode == KEYCODE_IC && key.key_mod == KEYCODE_SHIFT))
    {
      return_value = SH_PASS_KEY;
    }
    else
    {
     return_value = sh_process_keycode (key);
    }
  }
  else
  {
    /* Store the key */
    _nc_Buffer_Key.Push (key);
    /* Stop the key */
    return_value = SH_STOP_KEY;
  }

  if (return_value == SH_STOP_KEY || return_value == SH_SENT_KEY) {
    eng_stop_event(parm_event,'keypress');
    return false;
  }

  return true;
}

/*
  *  Function: eng_handle_up ()
  *
  *  Description: Last event fired.
  *
*/
function eng_handle_keyup(parm_event)
{
  var key_code;
  var return_value = false;

  trace ("eng_handle_keyup(parm_event)");

  if (parm_event.keyCode == KEYCODE_DC || (parm_event.keyCode == KEYCODE_BACKSPACE && is_field_input (parm_event.target)) || parm_event.keyCode == KEYCODE_CAPSLOCK) {
    return true;
  }
  else {
    eng_stop_event (parm_event,'keyup');
    return false;
  }
}

/*
  *  Function: eng_handle_focus ()
  *
  *  Description: Handle fields as they gain and
  *               lose focus.
  *
  * parm_event - event object
  * parm_this - should be a field element
  * parm_type - "focus" or "blur"
*/
function eng_handle_focus(parm_event, parm_this, parm_type)
{
  trace ("eng_handle_focus(parm_event, parm_this, "+parm_type+")");

  if (parm_type == "focus")
  {
    set_current_field (Main_form, parm_this);
    _nc_Refresh_Current_Field (Main_form);
    set_field_fore (current_field (Main_form), A_STANDOUT);
  }
  else if (parm_type == "blur") {
    /* Blur */
    _nc_Synchronize_Current_Field (Main_form);
    set_field_fore (current_field (Main_form), A_NORMAL);
  }
  else if (parm_type == "wfocus")
  {
    /* This is a focus event from the window */
    set_field_fore (current_field (Main_form), A_STANDOUT);
  }

  return true;
}

/*
  *  Function: eng_handle_click ()
  *
  *  Description: Handle fields when they are clicked
  *
  * parm_event - event object
  * parm_this - should be a field element
*/
function eng_handle_click(parm_event, parm_this)
{
  trace ("eng_handle_click(parm_event, parm_this)");

  set_field_fore (current_field (Main_form), A_STANDOUT);
  Sh_field_clear_flag = false;

  return true;
}