/*
*  JavaScript Implementation: sh
*
* Description: Screen Handler
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/


/***************************
*
*   GLOBAL SCREEN VARS
*
****************************/

/* Misc */
var Sh_field_clear_flag = false;       /* Clear field on first key */
var Sh_field_insert_flag = true;       /* Insert key toggle */
var Sh_drag_flag = false;
var Sh_screen_name;
var Sh_prev_screen;
var Sh_ctrl_mode = "0";
var Sh_first_key_blank = 0;
var Sh_zindex = 1;

/* Key mappings */
var Sh_config_parms = new Array();
var Sh_config_keys = new Array();

/* Windows */
var Sh_main_win;
var Sh_popup_win;
var Sh_tooltip_win;
var Sh_mesg_win;

/* Key mapping literals */
var F1_map = "F1";
var F2_map = "F2";
var F3_map = "F3";
var F4_map = "F4";
var F5_map = "F5";
var F6_map = "F6";
var F7_map = "F7";
var F8_map = "F8";
var F9_map = "F9";
var F10_map = "F10";
var F11_map = "F11";
var F12_map = "F12";
var F13_map = "F13";
var F14_map = "F14";
var F15_map = "F15";
var F16_map = "F16";
var F17_map = "F17";
var F18_map = "F18";
var F19_map = "F19";
var F20_map = "F20";
var F21_map = "F21";
var F22_map = "F22";
var F23_map = "F23";
var F24_map = "F24";
var ENT_map = "ENT";
var HOME_map = "HOME";
var END_map = "END";
var DOWN_map = "DOWN";
var UP_map = "UP";
var PGUP_map = "PGUP";
var PGDWN_map = "PGDWN";
var TAB_map = "TAB";
var ESC_map = "ESC";
var BACKSPACE_map = "BACKSPACE";

/* Window and form objects */
var Main_form;
var Sh_poll_read;               /* Timer handle for poll reads */
var Sh_get_seqn = 0;
var Sh_dual_key = 0;            /* Save key for dual key inputs */
var Sh_stop_tooltip_timer = false;
var Sh_tooltip_flag = OFF;
var Sh_find_son_timer = false;
var Sh_request_timer = false;
var Sh_request_max_time = 6000;
var Sh_req_for_son;
var Sh_http_errors = 0;
var Sh_flag_init = false;

var SH_ROW_PIXEL = 20;
var SH_COL_PIXEL = 11;

var SH_IO_READY = 0;
var SH_IO_NOTREADY = 1;
var SH_IO_READ = 2;
var SH_IO_WRITE = 3;
var SH_IO_WAIT = 4;

var SH_PASS_KEY = 0;          /* Let the browser handle key */
var SH_STOP_KEY = 1;          /* Stop the key and nothing */
var SH_SENT_KEY = 2;          /* Stop the key and I did something */
var Sh_IO_state = SH_IO_NOTREADY;

/* Vars used for draging popups around window */
var Sh_drag_offsetx;
var Sh_drag_offsety;
var Sh_drag_tempx;
var Sh_drag_tempy;

var ENG_SH_WT = "WT";

/*****************************
*
*    MAIN FUNCTIONS
*
******************************/
function eng_sh (parm_func,parm_data)
{
  trace ("eng_sh ("+parm_func+", "+parm_data+")");
  
  if (!Sh_flag_init) {
    sh_init ();
  }

  switch (parm_func)
  {
    case ENG_SH_WT:
    {
      sh_write(parm_data);
      break;
    }
  }
}

function sh_init ()
{
  var Sh_splash = "\n<form method=\"post\" name=\"main\" action=\"#\">\n<input type=\"text\" value=\"Initing Screen\">\n</form>";

  trace ("eng_sh_init ()");

  initscr ();                                                           /* Init curses */
  Sh_main_win = newwin (35, 98, 0, 0);
  Sh_popup_win = newwin (0, 0, 0, 0);
  Sh_tooltip_win = newwin (0, 0, 0, 0);
  Sh_mesg_win = newwin (2, 98, 34, 0);

  Sh_main_win.win.id = "main";
  Sh_popup_win.win.id = "popup";
  Sh_tooltip_win.win.id = "tooltip";
  Sh_mesg_win.win.id = "mesg";
  
  set_font_name (NC_FONT_NAME);
  set_font_size (NC_FONT_SIZE);

  Sh_main_win.win.innerHTML = Sh_splash;
  Main_form = new new_form("main");

  if (Sh_field_insert_flag) {
    form_driver (Main_form, REQ_INS_MODE);
  }
  else {
    form_driver (Main_form, REQ_OVL_MODE);
  }
  Sh_flag_init = true;
}

function sh_process_keycode (parm_key)
{
  var return_state = SH_STOP_KEY;
  var current_element = current_field(Main_form).element;

  if (parm_key.key_mod == KEYCODE_CTRL)
  {
    switch (parm_key.key)
    {
      case 69:      /* Ctrl-E-E Kill engine*/
      case 101:
      {
        if (Sh_dual_key == 69 || Sh_dual_key == 101)
        {
          Sh_dual_key = 0;
          sh_sub_keys("KL");
          unpost_form (Main_form);
          return_state = SH_SENT_KEY;
        }
        else
          Sh_dual_key = parm_key.key;
        break;
      }
      case 67:      /* Ctrl-C Clear message line */
      case 99:
      {
        sh_sub_keys("CM");
        return_state = SH_SENT_KEY;
        break;
      }
      case 82:      /* Ctrl-R Refresh screen */
      case 114:
      {
        sh_sub_keys("RW");
        return_state = SH_SENT_KEY;
        break;
      }
      case 113:    /* Ctrl-Q query the form for info */
     {
       alert ("Screen name: "+Sh_screen_name+" Field name: "+current_element.name);
       break;
     }
    }
  }
  else
  {
    if (parm_key.key >= 32 && parm_key.key <= 126)
    {
      if (Sh_field_clear_flag && Sh_first_key_blank)
      {
        form_driver (Main_form, REQ_CLR_FIELD);
        form_driver (Main_form, REQ_BEG_FIELD);
      }
      Sh_field_clear_flag = false;
      form_driver (Main_form, parm_key.key);
      set_field_fore (current_field (Main_form), A_STANDOUT);
    }
    else
    {
      switch (parm_key.key)
      {
        case KEY_F(1):
        {
          return_state = sh_sub_keys(check_mapping ("F1", F1_map));
          break;
        }
        case KEY_F(2):
        {
          return_state = sh_sub_keys(check_mapping ("F2", F2_map));
          break;
        }
        case KEY_F(3):
        {
          return_state = sh_sub_keys(check_mapping ("F3", F3_map));
          break;
        }
        case KEY_F(4):
        {
          return_state = sh_sub_keys(check_mapping ("F4", F4_map));
          break;
        }
        case KEY_F(5):
        {
          return_state = sh_sub_keys(check_mapping ("F5", F5_map));
          break;
        }
        case KEY_F(6):
        {
          return_state = sh_sub_keys(check_mapping ("F6", F6_map));
          break;
        }
        case KEY_F(7):
        {
          return_state = sh_sub_keys(check_mapping ("F7", F7_map));
          break;
        }
        case KEY_F(8):
        {
          return_state = sh_sub_keys(check_mapping ("F8", F8_map));
          break;
        }
        case KEY_F(9):
        {
          return_state = sh_sub_keys(check_mapping ("F9", F9_map));
          break;
        }
        case KEY_F(10):
        {
          return_state = sh_sub_keys(check_mapping ("10", F10_map));
          break;
        }
        case KEY_F(11):
        {
          return_state = sh_sub_keys(check_mapping ("11", F11_map));
          break;
        }
        case KEY_F(12):
        {
          return_state = sh_sub_keys(check_mapping ("12", F12_map));
          break;
        }
        case KEY_F(13):
        {
          return_state = sh_sub_keys(check_mapping ("13", F13_map));
          break;
        }
        case KEY_F(14):
        {
          return_state = sh_sub_keys(check_mapping ("14", F14_map));
          break;
        }
        case KEY_F(15):
        {
          return_state = sh_sub_keys(check_mapping ("15", F15_map));
          break;
        }
        case KEY_F(16):
        {
          return_state = sh_sub_keys(check_mapping ("16", F16_map));
          break;
        }
        case KEY_F(17):
        {
          return_state = sh_sub_keys(check_mapping ("17", F17_map));
          break;
        }
        case KEY_F(18):
        {
          return_state = sh_sub_keys(check_mapping ("18", F18_map));
          break;
        }
        case KEY_F(19):
        {
          return_state = sh_sub_keys(check_mapping ("19", F19_map));
          break;
        }
        case KEY_F(20):
        {
          return_state = sh_sub_keys(check_mapping ("02", F20_map));
          break;
        }
        case KEY_F(21):
        {
          return_state = sh_sub_keys(check_mapping ("21", F21_map));
          break;
        }
        case KEY_F(22):
        {
          return_state = sh_sub_keys(check_mapping ("22", F22_map));
          break;
        }
        case KEY_F(23):
        {
          return_state = sh_sub_keys(check_mapping ("23", F23_map));
          break;
        }
        case KEY_F(24):
        {
          return_state = sh_sub_keys(check_mapping ("24", F24_map));
          break;
        }
        case KEY_ENTER:
        {
          return_state = sh_sub_keys(check_mapping ("EN", ENT_map));
          break;
        }
        case KEY_ESC:
        {
          return_state = sh_sub_keys(check_mapping ("AT", ESC_map));
          break;
        }

        case KEY_HOME:
        {
          if (check_mapping (1, HOME_map) == 1)
            sh_home_field ();
          break;
        }

        case KEY_END:
        {
          if (check_mapping (1, END_map) == 1)
            sh_end_field ();
          break;
        }

        case KEY_DOWN:
        {
          if (!is_field_select (current_element))
          {
            if (check_mapping (1, DOWN_map) == 1)
              sh_down_field ();
          }
          else {
            return_state = SH_PASS_KEY;
          }
          break;
        }

        case KEY_UP:
        {
          if (!is_field_select (current_element))
          {
            if (check_mapping (1, UP_map) == 1)
              sh_up_field ();
          }
          else {
            return_state = SH_PASS_KEY;
          }
          break;
        }

        case KEY_LEFT:
        {
          if (is_field_input (current_element))
          {
            if (Sh_field_clear_flag)
            {
              Sh_field_clear_flag = false;
              set_field_fore (current_field (Main_form), A_STANDOUT);
              form_driver (Main_form, REQ_BEG_FIELD);
            }
            else
            {
              if (form_driver (Main_form, REQ_CAN_PREV_CHAR) == E_REQUEST_DENIED) {
                sh_prev_field ();
              }
              else
                return_state = SH_PASS_KEY;  /* Let the real key pass */
            }
          }
          else {
            sh_prev_field ();
          }
          break;
        }

        case KEY_RIGHT:
        {
          if (is_field_input (current_element))
          {
            if (Sh_field_clear_flag)
            {
              Sh_field_clear_flag = false;
              set_field_fore (current_field (Main_form), A_STANDOUT);
              form_driver (Main_form, REQ_END_FIELD);
            }
            else
            {
              if (form_driver (Main_form, REQ_CAN_NEXT_CHAR) == E_REQUEST_DENIED) {
                sh_next_field ();
              }
              else {
                return_state = SH_PASS_KEY;
              }
            }
          }
          else {
            sh_next_field ();
          }
          break;
        }

        case KEY_BACKSPACE:
        {
          set_field_fore (current_field (Main_form), A_STANDOUT);

          if (check_mapping (1, BACKSPACE_map) == 1)
          {
            if (is_field_input (current_element))
            {
              if (Sh_field_clear_flag && Sh_first_key_blank)
              {
                form_driver (Main_form, REQ_CLR_FIELD);
                form_driver (Main_form, REQ_BEG_FIELD);
              }
              else
              {
                if (form_driver (Main_form, REQ_CAN_PREV_CHAR) != E_OK)
                {
                  form_driver (Main_form, REQ_PREV_FIELD);
                  set_field_fore (current_field (Main_form), A_STANDOUT);
                  form_driver (Main_form, REQ_END_FIELD);
                  form_driver (Main_form, REQ_DEL_CHAR);
                }
                return_state = SH_PASS_KEY;
              }
            }
            else
            {
              form_driver (Main_form, REQ_PREV_FIELD);
              set_field_fore (current_field (Main_form), A_STANDOUT);
              form_driver (Main_form, REQ_END_FIELD);
              form_driver (Main_form, REQ_DEL_CHAR);
              return_state = SH_PASS_KEY;
            }
          }
          Sh_field_clear_flag = false;
          break;
        }

        case KEY_DC:                                      /* Delete */
        {
          set_field_fore (current_field (Main_form), A_STANDOUT);
          if (Sh_field_clear_flag && Sh_first_key_blank)
          {
            form_driver (Main_form, REQ_CLR_FIELD);
            form_driver (Main_form, REQ_BEG_FIELD);
          }
          Sh_field_clear_flag = false;
          return_state = SH_PASS_KEY;
          break;
        }

        case KEY_IC:                                      /* Insert */
        {
          set_field_fore (current_field (Main_form), A_STANDOUT);
          if (Sh_field_insert_flag) {
            form_driver (Main_form, REQ_OVL_MODE);
            Sh_field_insert_flag = false;
          }
          else {
            form_driver (Main_form, REQ_INS_MODE);
            Sh_field_insert_flag = true;
          }
          Sh_field_clear_flag = false;
          break;
        }

        case KEY_NPAGE:                                   /* Page Down */
        {
          if (check_mapping (1, PGDWN_map) == 1)
            clr_eof ();
          break;
        }

        case KEY_PPAGE:                                    /* PgUp */
        {
          if (check_mapping (1, PGUP_map) == 1)
            sh_next_field ();
          break;
        }

        case KEY_BTAB:
        {
          if (form_driver (Main_form, REQ_PREV_CHAR) == E_REQUEST_DENIED || Sh_field_clear_flag)
          {
            set_field_fore (current_field (Main_form), A_NORMAL);
            form_driver (Main_form, REQ_PREV_FIELD);
            select_field ();
          }
          else
          {
            form_driver (Main_form, REQ_BEG_FIELD)
          }
          break;
        }

        case KEY_TAB:
        {
          if (check_mapping (1, TAB_map) == 1) {
            sh_next_field ();
          }
          break;
        }

        default:
        {
          return_state = SH_PASS_KEY;       /* We did not handle the key, don't stop it */
          break;
        }
      } /* switch */
    } /* else */
  } /* else */

  if (Sh_dual_key != parm_key.key) {            /* Clear the dual key holder */
    Sh_dual_key = 0;
  }

  if (return_state != SH_SENT_KEY && Sh_ctrl_mode == "1")
  {
    sh_sub_keys(parm_key.charCode);
  }

  return return_state;
}

function check_mapping (pressed_key, key)
{
  var rtn_key;
  var x = 0;

  trace ("check_mapping ("+pressed_key+", "+key+")");
  
  rtn_key = pressed_key;

  while (Sh_config_keys[x])
  {
    if (Sh_config_keys[x] == key)
    {
      trace ("check_mapping: Got map for "+Sh_config_keys[x]);
      if (Sh_config_keys[x+1].substr(0,4) == "KEY_")
      {
        rtn_key = Sh_config_keys[x+1].substring(4);
        break;
      }
      else
      {
        if (Sh_config_keys[x+1] == "REQ_CLR_EOS")
        {
          clr_eos ();
          rtn_key = false;
        }
        else if (Sh_config_keys[x+1] == "REQ_CLR_EOL")
        {
          clr_eol ();
          rtn_key = false ;
        }
        else if (Sh_config_keys[x+1] == "REQ_CLR_EOF")
        {
          clr_eof ();
          rtn_key = false ;
        }
        else if (Sh_config_keys[x+1] == "REQ_NEXT_FIELD")
        {
          next_field ();
          rtn_key = false ;
        }
        else if (Sh_config_keys[x+1] == "REQ_PREV_FIELD")
        {
          prev_field ();
          rtn_key = false ;
        }
        else if (Sh_config_keys[x+1] == "REQ_FIRST_FIELD")
        {
          home_field ();
          rtn_key = false ;
        }
        else if (Sh_config_keys[x+1] == "REQ_DOWN_FIELD")
        {
          down_field ();
          rtn_key = false ;
        }
        else if (Sh_config_keys[x+1] == "REQ_LEFT_CHAR")
        {
          left_char ();
          rtn_key = false ;
        }
        break;
      }
    }
    x = x + 2;
  }
  return (rtn_key);
}

function clr_eos()
{
  trace ("clr_eos ()");
  
  set_field_fore (current_field (Main_form), A_NORMAL);
  if (Sh_field_clear_flag)
    form_driver (Main_form, REQ_BEG_FIELD);
  form_driver (Main_form, REQ_CLR_EOS);
  form_driver (Main_form, REQ_NEXT_FIELD);
  select_field ();
}

function clr_eol()
{
  trace ("clr_eol ()");
  
  set_field_fore (current_field (Main_form), A_NORMAL);
  if (Sh_field_clear_flag)
    form_driver (Main_form, REQ_BEG_FIELD);
  form_driver (Main_form, REQ_CLR_EOL);
  form_driver (Main_form, REQ_NEXT_FIELD);
  select_field ();
}

function clr_eof()
{
  trace ("clr_eof ()");
  
  set_field_fore (current_field (Main_form), A_NORMAL);
  if (Sh_field_clear_flag)
    form_driver (Main_form, REQ_BEG_FIELD);
  form_driver (Main_form, REQ_CLR_EOF);
  form_driver (Main_form, REQ_NEXT_FIELD);
  select_field ();
}

function sh_down_field()
{
  trace ("sh_down_field ()");

  set_field_fore (current_field (Main_form), A_NORMAL);
  form_driver (Main_form, REQ_DOWN_FIELD);
  select_field ();
}

function sh_up_field()
{
  trace ("sh_up_field ()");

  set_field_fore (current_field (Main_form), A_NORMAL);
  form_driver (Main_form, REQ_UP_FIELD);
  select_field ();
}

function sh_home_field()
{
  trace ("sh_home_field ()");
  
  set_field_fore (current_field (Main_form), A_NORMAL);
  form_driver (Main_form, REQ_FIRST_FIELD);
  select_field ();
}

function sh_end_field()
{
  trace ("sh_end_field ()");

  set_field_fore (current_field (Main_form), A_NORMAL);
  form_driver (Main_form, REQ_DOWN_FIELD);
  select_field ();
}

function sh_next_field()
{
  trace ("sh_next_field ()");

  set_field_fore (current_field (Main_form), A_NORMAL);
  form_driver (Main_form, REQ_NEXT_FIELD);
  select_field ();
}

function sh_prev_field()
{
  trace ("sh_prev_field ()");

  set_field_fore (current_field (Main_form), A_NORMAL);
  form_driver (Main_form, REQ_PREV_FIELD);
  select_field ();
}

function left_char ()
{
  var return_val;

  trace ("left_char ()");

  if (form_driver (Main_form, REQ_PREV_CHAR) == E_REQUEST_DENIED)
  {
    sh_prev_field();
  }
}

function select_field ()
{
  var current_element = current_field(Main_form).element;

  trace ("select_field ()");

  if (is_field_input (current_element)) {
    set_field_fore (current_field (Main_form), A_SELECT);
  }
  else {
    set_field_fore (current_field (Main_form), A_STANDOUT);
  }
  Sh_field_clear_flag = true;
}


/*
*  Function sh_change_background ()
*
*
*/
function sh_change_background(parm_this,parm_color)
{
  trace ("sh_change_background ("+parm_this.name+", "+parm_color+")");

  parm_this.style.background = parm_color;
}

/*
*  Function sh_button_click ()
*
*  Description: Update the field in parm_id with parm_value and fire
*  the key in parm_key.
*
*/
function sh_button_click(parm_value,parm_key,parm_id)
{
  trace ("sh_button_click ("+parm_value+", "+parm_key+", "+parm_id+")");
  
  if (parm_value != "" && parm_value != " ") {
    set_field_buffer (field_by_name(Main_form,parm_id),0,parm_value);
  }
  if (parm_key != " ") {
    sh_sub_keys(parm_key);
  }
}

/*
*  Function sh_list_select ()
*
*  Description: Update the field in parm_id with parm_value and fire
*  the key in parm_key.
*
*/
function sh_list_select (parm_key)
{
  trace ("sh_list_select ("+parm_key+")");

  if (parm_key != " " && parm_key != "") {
    sh_sub_keys(parm_key);
  }
  return true;
}


/*
*  Function sh_init_tooltip ()
*
*
*/
function sh_init_tooltip(parm_event,parm_this,parm_tip)
{
  var current_scr = Sh_tooltip_win.win;

  trace ("sh_init_tooltip (event, this, "+parm_tip+")");
  
  if (!current_scr)
  return;

  if (parm_tip == " " || parm_tip == "")
  return;
  
  if (!parm_event) {
    parm_event = window.event;
  }

  if (Sh_stop_tooltip_timer) {
    clearTimeout(Sh_stop_tooltip_timer);
    Sh_stop_tooltip_timer = "";
  }
  current_scr.style.border='1px solid #A5CFE9';
  current_scr.style.width='300';
  current_scr.style.fontFamily='arial';
  current_scr.style.fontSize='12';
  current_scr.style.padding='3';
  current_scr.style.color='#1B4966';
  current_scr.style.background='#FFFFFF';
  current_scr.style.filter='alpha(opacity=85)'; // IE
  current_scr.style.opacity='0.85'; // FF
  current_scr.style.left = parm_event.clientX + SH_COL_PIXEL;
  current_scr.style.top = parm_event.clientY - SH_ROW_PIXEL;
  current_scr.style.position = 'absolute';
  current_scr.style.visibility = 'visible';

  current_scr.innerHTML = parm_tip;

  Sh_drag_offsetx = parm_event.clientX;
  Sh_drag_offsety = parm_event.clientY;
  
  Sh_drag_tempx = parseInt(current_scr.style.left);
  Sh_drag_tempy = parseInt(current_scr.style.top);
  
  this.onmousemove = sh_drag_tooltip;

  Sh_tooltip_flag = ON;
}

/*
*  Function sh_stop_tooltip ()
*
*
*/
function sh_stop_tooltip()
{
  trace ("sh_stop_tooltip ()");
  
  if (!Sh_tooltip_win.win)
  return;

  if (Sh_tooltip_flag == OFF)
  return;

  Sh_tooltip_flag = HALF;
  this.onmousemove = null;
  
  var opacity = parseFloat(Sh_tooltip_win.win.style.opacity);
  opacity -= parseFloat('0.05');
  if (opacity > 0)
  {
    Sh_tooltip_win.win.style.opacity = opacity;
    Sh_stop_tooltip_timer = setTimeout("sh_stop_tooltip()",200);
    return;
  }
  
  Sh_tooltip_win.win.style.visibility = 'hidden';
  Sh_tooltip_win.win.innerHTML = "";

  Sh_tooltip_flag = OFF;
}

/*
*  Function sh_drag_tooltip ()
*
*
*/
function sh_drag_tooltip(e)
{
  var current_scr = Sh_tooltip_win.win;
  var button;

  trace ("sh_drag_tooltip (event)");
  
  if (!current_scr) {
    return;
  }

  if (Sh_tooltip_flag != ON) {
    return;
  }
  
  if (!e)
  {
    if (window.event) {
      e = window.event;
    }
    else if (event) {
      e = event;
    }
    else {
      return false;
    }
  }

  /* IE hase buttons */
  if (e.button) {
    button = e.button;
  }
  else {
    button = 1;
  }

  if (button == 1)
  {
    current_scr.style.left = Sh_drag_tempx + e.clientX - Sh_drag_offsetx + "px";
    current_scr.style.top = Sh_drag_tempy + e.clientY - Sh_drag_offsety + "px";
  }
}

/*
*  Function sh_init_drag ()
*
*
*/
function sh_init_drag(parm_event)
{
  var current_scr = document.getElementById("current_scr");

  if (!parm_event) {
    parm_event = window.event;
  }

  trace ("sh_init_drag (event[tagName:"+parm_event.target.tagName+"])");

  if (!current_scr)
  return true;

  if (parm_event.target.tagName != "DIV") {
    return true;
  }

  Sh_tooltip_win.win.style.visibility = 'hidden';
  Sh_tooltip_win.win.innerHTML = "";

  Sh_drag_offsetx = parm_event.clientX;
  Sh_drag_offsety = parm_event.clientY;

  Sh_drag_tempx = parseInt(current_scr.style.left);
  Sh_drag_tempy = parseInt(current_scr.style.top);

  current_scr.onmousemove = sh_drag_drop;
  Sh_drag_flag = true;
}

/*
*  Function sh_drag_drop ()
*
*
*/
function sh_drag_drop(e)
{
  var current_scr = document.getElementById("current_scr");
  var button;

  trace ("sh_drag_drop (event)");
  
  if (!current_scr) {
    return;
  }

  if (!e)
  {
    if (window.event) {
      e = window.event;
    }
    else if (event) {
      e = event;
    }
    else {
      return false;
    }
  }

  /* IE hase buttons */
  if (e.button) {
    button = e.button;
  }
  else {
    button = 1;
  }

  if (button == 1)
  {
    current_scr.style.left = Sh_drag_tempx + e.clientX - Sh_drag_offsetx + "px";
    current_scr.style.top = Sh_drag_tempy + e.clientY - Sh_drag_offsety + "px";
  }
}

/*
*  Function sh_stop_drag ()
*
*
*/
function sh_stop_drag()
{
  var current_scr = document.getElementById("current_scr");

  trace ("sh_stop_drag ()");
  
  if (!current_scr || !Sh_drag_flag)
  return;

  current_scr.onmousemove = null;
  Sh_drag_flag = false;
}


/*
*  Function: sh_write ()
*
*  Description: Write out data to webserver
*
*  Notes: Every write to the server sets a listener
*         via XMLHttpRequest.
*/
function sh_write (data)
{
  trace ("sh_write: ("+data+")");

  Sh_IO_state = SH_IO_WRITE;

  var request_handle = sh_get_XMLHttpRequest();
  if (!request_handle)
  {
    alert("Browser not Supported");
    return;
  }

  if (Sh_request_timer)
  {
    clearTimeout(Sh_request_timer);
    Sh_request_timer = false;
  }

  /* Setup Read hook for this Write*/
  var handlerFunction = sh_read (request_handle);
  request_handle.onreadystatechange = handlerFunction;

  request_handle.open ('POST', Server_url, true);

  Sh_request_timer = setTimeout(function() {
    request_handle.abort();
    sh_write (data);
  }, Sh_request_max_time);

  request_handle.setRequestHeader("Content-Type", "text/html");
  request_handle.send (data);

  if (data.substr(0,7) == "!#GET#!") {
      Sh_get_seqn++;
  }
}


/*
*  Function: sh_read_poll ()
*
*  Description: Ask for a web page using GET
*
*/
function sh_read_poll ()
{
  trace ("sh_read_poll ()");

  /* Cancel any pending polls */
  if (Sh_poll_read) {
    clearTimeout(Sh_poll_read);
    Sh_poll_read = false;
  }

  if (Sh_request_timer)
  {
    clearTimeout(Sh_request_timer);
    Sh_request_timer = false;
  }

  Sh_IO_state = SH_IO_READ;

  var request_handle = sh_get_XMLHttpRequest();
  if (!request_handle)
  {
    alert("Browser not Supported");
    return;
  }

  /* Setup Read hook for this Read */
  var handlerFunction = sh_read (request_handle);
  request_handle.onreadystatechange = handlerFunction;

  request_handle.open ('GET', Server_url, true);

  Sh_request_timer = setTimeout(function() {
    request_handle.abort();
    sh_read_poll ();
  }, Sh_request_max_time);

  request_handle.setRequestHeader("Content-Type", "text/html");
  request_handle.send ("");
}

/*
* Function: find_son ()
*
* Description: Bug the engine every sec for a page
*
*/
function find_son ()
{
  trace ("find_son ()");

  if (Sh_find_son_timer)
   Sh_req_for_son.abort ();

  Sh_req_for_son = sh_get_XMLHttpRequest();
  var handlerFunction = sh_read (Sh_req_for_son);
  Sh_req_for_son.onreadystatechange = handlerFunction;

  Sh_req_for_son.open ('GET', Server_url, true);
  Sh_req_for_son.setRequestHeader("Content-Type", "text/html");
  Sh_req_for_son.send ("");
  Sh_find_son_timer = setTimeout("find_son ();", 1500); 
}

/*
*  Function: sh_read ()
*
*  Description: Read data from webserver
*
*/
function sh_read(request_handle)
{
  return function ()
  {
    try
    {
      var req_status;

      trace ("sh_read (request_handle[readyState: "+request_handle.readyState+"])");

      /* Cancel any pending polls */
      if (Sh_poll_read) {
        clearTimeout(Sh_poll_read);
        Sh_poll_read = false;
      }

      if (Sh_request_timer && request_handle.readyState > 1)
      {
        clearTimeout(Sh_request_timer);
        Sh_request_timer = false;
      }

      Sh_IO_state = SH_IO_READ;

      if (request_handle.readyState == 4)
      {
        /* Cancel any pending son searching */
        if (Sh_find_son_timer)
        {
          clearTimeout(Sh_find_son_timer);
          Sh_find_son_timer = false;
        }

        try {
          req_status = request_handle.status
        } catch (e) {
          /* If an abort is issued status becomes
             unavailabe */
          req_status = 999;
        }

        if (req_status == 200)
        {
          /* Since javascript passes by value, for performance sake we are
          not going to pass the responseText to a function cause it would
          have to copy the entire web page to the function */


          /****** MAIN READ *****/
          var screen_type;

          trace ("sh_read: process HTTP Started");

          Sh_http_errors = 0;

          /* Check for command */
          if (request_handle.responseText.substr(0,7) == "!#CMD#!")
          {
            sh_process_command (request_handle.responseText);
          }
          else
          {
          /* If the engine sent back a valid response we should have
            a header with various info */

            trace ("sh_read: Got WebPage: "+request_handle.responseText.substr(0,20));

            /* Kill any tool tips fading */
            Sh_tooltip_win.win.style.visibility = 'hidden';

            screen_type = request_handle.responseText.substr(14,1);

            /* Reg screen */
            if (screen_type == 0)
            {
              var current_scr = document.getElementById("current_scr");

              Sh_prev_screen = Sh_screen_name;
              Sh_screen_name = request_handle.responseText.substr(15,8);

              if (current_scr) {
                /* Disable any current screens */
                current_scr.id = "blah";
              }

              /* Reg screens close all the popups */
              Sh_popup_win.win.innerHTML = "";

              /* This moves the received webpage to the screen */
              Sh_main_win.win.innerHTML = request_handle.responseText;

              document.body.style.background = document.getElementById("current_scr").style.background;

              /* Setup form */
              free_form (Main_form);
              Main_form = new new_form("main");
              post_form (Main_form);
              Sh_field_clear_flag = false;
              Sh_zindex = 1;
            }
            else if (screen_type == 1)
            {
              /* Popup screen */
              var current_scr = document.getElementById("current_scr");
              var prev_popup;

              Sh_screen_name = request_handle.responseText.substr(15,8);
              
              if (Sh_screen_name != Sh_prev_screen)
              {
                unpost_form (Main_form);
                
                /* Dump old popup to main */
                current_scr.style.zIndex = Sh_zindex;
                Sh_zindex += 100;
                
                prev_popup = document.getElementById(Sh_prev_screen);
                if (prev_popup) {
                  Sh_main_win.win.replaceChild(current_scr, prev_popup);
                }
                else {
                  Sh_main_win.win.appendChild(current_scr);
                }
                //Sh_main_win.win.innerHTML = Sh_main_win.win.innerHTML + Sh_popup_win.win.innerHTML;
                
               /* Give old screen its name */
                current_scr.id = Sh_prev_screen;
              }

              Sh_prev_screen = Sh_screen_name;
              
              /* Display new popup */
              Sh_popup_win.win.innerHTML = request_handle.responseText;

              /* Setup form */
              free_form (Main_form);
              Main_form = new new_form("main");
              post_form (Main_form);
              Sh_field_clear_flag = false;
            }
            else if (screen_type == 2)
            {
              /* Message screen */
              Sh_mesg_win.win.innerHTML = request_handle.responseText;
            }
            else if (screen_type == 3)
            {
              /* Alert screen */
              alert(request_handle.responseText.substring(37));
            }
            else
            {
              /* Must be an early on error */
              alert(request_handle.responseText);
            }

            /* Set focus on a field */
            sh_set_focus ();

            sh_write ("!#GET#!"+Sh_get_seqn);
          }
          /************ END OF STATUS 200 ***********/
        }
        else
        {
          /* This is for a non-200 status */
          /* This always happens when a EXEC/TEST returns to root prog */
          trace ("sh_read: HTTP error - data: "+request_handle.responseText+"\nheaders: "+request_handle.getAllResponseHeaders());
          if (Sh_http_errors++ == 10)
          {
           alert("sh_read: HTTP error");
          }
          else
          {
            sh_read_poll ();                /* Try to read again */
          }
        }
      }
    } catch(e) {                                              /* This happens on an req.abort (usually) */
      trace ("sh_read: Exception: " + e);
    }  /* try/catch */
  } /* function */
}

/* Send keys to the server */
function sh_sub_keys(keys)
{
  /* Try to stop any other keys as soon as possible */
  Sh_IO_state = SH_IO_NOTREADY;

  var current_element =  current_field (Main_form).element;
  var sel_start;

  trace ("sh_sub_keys("+keys+")");

  if (!keys)
  {
    Sh_IO_state = SH_IO_READY
    keys = SH_STOP_KEY
  }
  else
  {
    set_field_fore (current_field (Main_form), A_NORMAL);

    /* Be sure we Sync the page data to the form data */
    _nc_Synchronize_Current_Field (Main_form);

    /* Update the key element with the key pressed */
    set_field_buffer (Main_form.field[Main_form.keys_element], 0, keys);

    /* When calculating the cursor poistion, cursor is only valid in input fields */
    if (is_field_input (current_element)) {
      sel_start = current_element.selectionStart;
    }
    else {
      sel_start = 0;
    }

    /* Return the current cursor position */
    if (Main_form.cursor_pos_element > -1)
    {
      set_field_buffer (Main_form.field[Main_form.cursor_pos_element], 0, leading_zero((parseInt (current_element.style.top) / NC_ROW_PIXEL)) + leading_zero((parseInt (current_element.style.left) / NC_COL_PIXEL) + (parseInt (sel_start) + 1)));
    }

    /* Returns all values from the "main" form */
    sh_write (sh_extract_form_data(Main_form));
    keys = SH_SENT_KEY;
  }
  return (keys);
}

function sh_set_focus()
{
  var nfocus;
  var tfocus;
  var xdisp;

  trace ("sh_set_focus()");

  try
  {
    /* This is the name of the field, which is also the x-y cordinates */
    if (Main_form.focus_name_element < 0 || Main_form.cursor_pos_element < 0) {
      return;
    }
    nfocus = Main_form.field[Main_form.focus_name_element].buf;

    if (!nfocus || nfocus == " ") {
      return;
    }

    /* Get x-cordinate cursor position, since setSelectionRange is offset from input box
    and what we get is offset from window, we get the difference */
    trace ("sh_set_focus: cur_pos:"+Main_form.field[Main_form.cursor_pos_element].buf+" focus_name:"+Main_form.field[Main_form.focus_name_element].buf);
    xdisp = Main_form.field[Main_form.cursor_pos_element].buf.substring(2) - Main_form.field[Main_form.focus_name_element].buf.substr(2,2);

    if (xdisp == "NaN") {
      return;
    }

    trace ("sh_set_focus: xdisp: "+xdisp);
    if (nfocus && nfocus != " ")
    {
      tfocus = document.getElementById(nfocus);
      if (tfocus)
      {
        tfocus.focus ();
        if (is_field_input(tfocus)) {
          /* Move to the real curser position */
          tfocus.setSelectionRange(xdisp, xdisp); 
        }
        set_field_fore (field_by_name (Main_form, nfocus), A_STANDOUT);
      }
      else
      {
        if (document.forms["main"].elements[0]) {
          document.forms["main"].elements[0].focus();
        }
      }
    }
    else
    {
      if (document.forms["main"].elements[0]) {
        document.forms["main"].elements[0].focus();
      }
    }
  } catch (e)
  {
    trace ("sh_set_focus: ERROR:"+e);
  }
}

/*
*  Function: sh_extract_form_data ()
*
*  Description: Extract all the data from the form
*
*/
function sh_extract_form_data (parm_form)
{
  var getstr = "";
  var real_form = parm_form.form;
  var real_element;

  trace ("sh_extract_form_data ("+parm_form.name+")");
  
  for (i=0; i < real_form.length; i++)
  {
    real_element = real_form.elements[i];
    if (real_element.tagName == "INPUT")
    {
      if ((is_field_input (real_element) ||
      (real_element.type == "hidden")) && !real_element.disabled)
      {
        getstr += parm_form.field[i].buf;
        for (j = parm_form.field[i].length; j < parm_form.field[i].maxgrow; j++) {
          getstr += " ";
        }
      } else if (real_element.type == "checkbox")
      {
        if (real_element.checked)
        {
          getstr += "Y";
        } else
        {
          getstr += "N";
        }
      }
    } else if (real_element.tagName == "SELECT")
    {
      if (real_element.selectedIndex >= 0)
      {
        getstr += real_element.options[real_element.selectedIndex].value;
      }
      else
      {
        for (j = 0; j < parm_form.field[i].maxgrow; j++) {
          getstr += " ";
        }
      }
    }
  }
  return getstr;
}


/*
*  Function: sh_process_command ()
*
*  Description: Process commands from host
*
* Current opcodes:
* 01 - Engine is awaiting data
* 02 - Engine has forked a child
* 03 - Sync up with ui
* 04 - Engine has just started needs handshake
* 05 - Engine ended
* 06 - Get/set font settings
*
*/
function sh_process_command (command)
{
  var opcode;
  var OPCODE = 7;
  var OPERANDS = 9;

  /* Format of a command is !#CMD#!??$$$$.. */
  opcode = parseInt (command.substr(OPCODE,2));

  trace ("sh_process_command: Got opcode: "+opcode);
  switch (opcode)
  {
    case 1:
    {
      trace ("sh_process_command: Got READ mode: "+command.substring(OPERANDS));

      Sh_ctrl_mode = command.substring(OPERANDS);

      /* The engine is read for input so check to see if
      we have buffered any of the users keystrokes */
      while (_nc_Buffer_Key.Have_key ())
      {
        /* If submitted a key, stop */
        if (sh_process_keycode (_nc_Buffer_Key.Pop ()) == SH_SENT_KEY) {
          return;
        }
      }

      Sh_IO_state = SH_IO_READY;

      /* This means the engine is waiting for us but we
      are going to bug the engine every  5 seconds */
      Sh_poll_read = setTimeout("sh_write (\"!#GET#!\"+Sh_get_seqn);", 5000);
      break;
    }

    case 2:
    {
      trace ("sh_process_command: Got FORK");

      /* The engine has exec/test so I need to read
      until I recieve a screen from son */
      find_son ();
      break;
    }

    case 3:
    {
      trace ("sh_process_command: Got SYNC");

      /* Just wait for engine */
      sh_read_poll ();
      break;
    }

    case 4:
    {
      var i,j;

      trace ("sh_process_command: Got " + command);

      /* The engine just started so we must handshake */
      Sh_config_parms = command.substring(OPERANDS).split(':');
      for (i=0; Sh_config_parms[i]; i++)
      {
        trace ("sh_read: Sh_config_parms["+i+"] "+Sh_config_parms[i]);
        if (Sh_config_parms[i].substr(0,6) == "!KEYS!")
        {
          Sh_config_keys = Sh_config_parms[i].substring(7).split(' ');
          j=0;
          while (Sh_config_keys[j])
          {
            j += 2;
          }
        }
        else if (Sh_config_parms[i].substr(0,16) == "first_key_blank=")
        {
          Sh_first_key_blank = parseInt (Sh_config_parms[i].substring(16));
        }
      }
      sh_write("!#INFO#!:BROWSER_OS="+NC_BROWSER_OS+":BROWSER_TYPE="+NC_BROWSER_TYPE+":FONT_NAME="+NC_FONT_NAME+":FONT_SIZE="+NC_FONT_SIZE+":ROW_PIXEL="+NC_ROW_PIXEL+":COL_PIXEL="+NC_COL_PIXEL);
      break;
    }

    case 5:
    {
      trace ("sh_process_command: Got END");
      sh_exit ();
      break;
    }

    case 6:
    {
      trace ("sh_process_command: Got FONT");
      var sub_op = parseInt (command.substr(OPERANDS,2));
      var new_value = command.substr(OPERANDS+2,2);

      switch (sub_op)
      {
        case 1:
        {
          /* Set Font name */
          set_font_name (new_value);
          break;
        }
        case 2:
        {
          /* Set Font size */
          set_font_size (new_value);
          break;
        }
        case 3:
        {
          /* Set row pixel size */
          set_row_pixel (new_value);
          break;
        }
        case 4:
        {
          /* Set col pixel size */
          set_col_pixel (new_value);
          break;
        }
      } /*Switch */

      sh_write ("!#GET#!"+Sh_get_seqn);
      break;
    }

    default:
    {
      /* Unknown command */
      trace ("sh_process_command: unknown command: " + command);
      sh_read_poll ();
      break;
    }
  }
}


/*
*  Function: sh_exit ()
*
*  Description: Exit the sh handler
*
*
*/
function sh_exit ()
{
  trace ("sh_exit ()");

  free_form (Main_form);
  Sh_main_win.win.innerHTML = "<a href=\""+window.location.search+"\">Smart System "+Terminal_id+"</a>";
  Sh_popup_win.win.innerHTML = "";
  Sh_tooltip_win.win.innerHTML = "";
}


/*
*  Function: sh_get_XMLHttpRequest ()
*
*  Description: Returns a new XMLHttpRequest object, or false if this browser
*               doesn't support it.
*
*/
function sh_get_XMLHttpRequest()
{
  var xmlreq = false;

  trace ("sh_get_XMLHttpRequest ()");

//   try {
//     netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserRead");
//   } catch (e) {
//     alert("Permission UniversalBrowserRead denied.");
//   }

  if (window.XMLHttpRequest)
  {
    xmlreq = new XMLHttpRequest();      /* Create XMLHttpRequest object in non-Microsoft browsers */
  }
  else if (window.ActiveXObject)
  {
    try
    {
      xmlreq = new ActiveXObject("Msxml2.XMLHTTP");       /* Windows ActiveX */
    } catch (e1) {
      try                               /* Failed to create required ActiveXObject */
      {
        xmlreq = new ActiveXObject("Microsoft.XMLHTTP");      /* Older windows */
      } catch (e2) {
        /* Unable to create an XMLHttpRequest with ActiveX */
      }
    }
  }
  return xmlreq;
}

