/*
*  JavaScript Implementation: jscurses
*
* Description:
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/


/*
*  Form actions driver
*
*/
function form_driver (parm_form, parm_action)
{
  if (parm_form)
  {
    if ((parm_form.status & _POSTED) != _POSTED)
      return;
  }

  var current_element = current_field(parm_form).element;
  var real_form = parm_form.form;
  var i;
  var got_focus;
  var return_val = E_OK;

  trace ("form_driver ("+parm_form.name+", "+parm_action+")");

  /* See if it's an actual ascii character */
  if (parm_action > 0 && parm_action < 256)
  {
    var chr = String.fromCharCode(parm_action);

    if (is_field_input (current_element))
    {
      var start_pos = current_element.selectionStart;
      var end_pos = current_element.selectionEnd;
      var end_adj = 0;

      if (!NC_INSERT_TOGGLE)
      {
        /* Ovr mode */
        if (end_pos < current_element.value.length)
          end_adj = 1;
      }

      if (current_element.size == current_element.maxLength)
      {
        current_element.value=current_element.value.substring(0,start_pos)+chr+current_element.value.substring(end_pos+end_adj,current_element.value.length);

        if ((start_pos + 1) >= current_element.maxLength) {
          next_field (parm_form);
        }
        else {
          set_cursor (current_element, start_pos + 1);
        }
      }
      else {
        fire_keycode (current_element,parm_action,2);     /* On overloaded field we have to fire the key so it will sroll left */
      }
    }
    else if (current_element.type == "checkbox")
    {
      if (current_element.checked)
      {
        current_element.checked = false;
      } else
      {
        current_element.checked = true;
      }
    }
    else if (current_element.type == "button")
    {
      /* 32=SpaceBar */
      if (parm_action == 32) {
        fire_click (current_element);
      }
    }
  }
  else
  {
    switch (parm_action)
    {
      case REQ_PREV_FIELD:            /* Focus on previous element */
      {
        prev_field (parm_form);
        break;
      }

      case REQ_NEXT_FIELD:            /* Focus on next element */
      {
        next_field (parm_form);
        break;
      }

      case REQ_UP_FIELD:
      {
        var current_row = parm_form.field[parm_form.current].top;
        var current_row_upper = current_row - 1;
        var current_row_lower = current_row + 1;
        var current_col = parm_form.field[parm_form.current].left;
        var pos = 0;
        var sav_col = 100;
        var diff_col = 0;

        /* Walk backwards thru the form elements till the row is different */
        for (pos=(parm_form.current-1); pos >= 0 && parm_form.field[pos].top > current_row_upper && parm_form.field[pos].top < current_row_lower && current_col != parm_form.field[pos].left; pos--);

        /* loop back to the last field on screen if more than two lines */
        if (pos < 0 && current_row != parm_form.field[real_form.length - 1].top)
        {
          /* Backwards to last non-hidden field */
          for (pos = (real_form.length - 1); pos >= 0  && (real_form.elements[pos].type == "hidden" || real_form.elements[pos].disabled);pos--);
          real_form.elements[pos].focus();
        }
        else
        {
          /* Backwards to non-hidden field */
          for (; pos >= 0  && (real_form.elements[pos].type == "hidden" || real_form.elements[pos].disabled) ;pos--);

          /* Now try to find the closest field above the orig field */
          current_row = parm_form.field[pos].top;
          while (pos >= 0 && current_row == parm_form.field[pos].top)
          {
            if (parm_form.field[pos].left > current_col)
            {
              diff_col = parm_form.field[pos].left - current_col;
            }
            else
            {
              diff_col = current_col - parm_form.field[pos].left;
            }
            if (diff_col < sav_col)
            {
              sav_col = diff_col;
            }
            else
            {
              break;
            }
            pos--;
          }
          real_form.elements[++pos].focus();
        }
        begin_field (real_form.elements[pos]);        /* Set Curser to beg of field */
        break;
      }

      case REQ_DOWN_FIELD:
      {
        var current_row = parm_form.field[parm_form.current].top;
        var current_row_upper = current_row - 1;
        var current_row_lower = current_row + 1;
        var pos = 0;

        /* Forward to first non-hidden field on different row */
        for (pos=parm_form.current; pos < real_form.length && (real_form.elements[pos].type == "hidden" || parm_form.field[pos].top == current_row || real_form.elements[pos].disabled); pos++);

        if (pos < real_form.length)
        {
          real_form.elements[pos].focus();

          /* Set Curser to beg of field */
          begin_field (real_form.elements[pos]);
        }
        else {
          /* Loop back to the top */
          first_field (real_form);
        }
        break;
      }

      case REQ_FIRST_FIELD:
      {
        /* Forward to first non-hidden field */
        first_field (real_form);
        break;
      }

      case REQ_NEXT_CHAR:
      {
        if (is_field_input (current_element))
        {
          var end_pos = current_element.selectionEnd;

          if (end_pos >= (current_element.maxLength - 1))
          {
            return_val = E_REQUEST_DENIED;
          }
          else
          {
            set_cursor (current_element, end_pos + 1);
          }
        }
        else
          return_val = E_REQUEST_DENIED;
        break;
      }

      case REQ_CAN_NEXT_CHAR:
      {
        if (is_field_input (current_element))
        {
          var end_pos = current_element.selectionEnd;

          if (end_pos >= (current_element.maxLength - 1))
            return_val = E_REQUEST_DENIED;
        }
        else
          return_val = E_REQUEST_DENIED;
        break;
      }

      case REQ_PREV_CHAR:
      {
        if (is_field_input (current_element))
        {
          var end_pos = current_element.selectionEnd;
          if (end_pos == 0)
          {
            return_val =E_REQUEST_DENIED;
          }
          else
          {
            set_cursor (current_element, end_pos - 1);
          }
        }
        else
          return_val = E_REQUEST_DENIED;
        break;
      }

      case REQ_CAN_PREV_CHAR:
      {
        if (is_field_input (current_element))
        {
          var end_pos = current_element.selectionEnd;
          if (end_pos == 0)
            return_val =E_REQUEST_DENIED;
        }
        else
          return_val = E_REQUEST_DENIED;
        break;
      }

      case REQ_DEL_CHAR:
      {
        if (is_field_input (current_element))
        {
          var start_pos = current_element.selectionStart;

          current_element.value=current_element.value.substring(0,start_pos)+current_element.value.substring(start_pos+1);
          set_cursor (current_element, start_pos);
        }
        else if (current_element.type == "checkbox")
        {
          current_element.checked = false;
        }
        break;
      }

      case REQ_CLR_FIELD:
      {
        if (is_field_input (current_element))
        {
          current_element.value = "";
        }
        else if (current_element.type == "checkbox")
        {
          current_element.checked = false;
        }
        break;
      }

      case REQ_CLR_EOF:
      {
        clear_eof (current_element);
        break;
      }

      case REQ_CLR_EOL:
      {
        var current_row = parm_form.field[parm_form.current].top;

        clear_eof (current_element);
        /* Forward till field on different row */
        for (i=parm_form.current + 1; i < real_form.length && parm_form.field[i].top == current_row; i++)
        {
          if (is_field_input (real_form.elements[i]))
          {
            real_form.elements[i].value = "";
          }
          else if (real_form.elements[i].type == "checkbox")
          {
            real_form.elements[i].checked = false;
          }
        }
        real_form.elements[i-1].focus();
        break;
      }

      case REQ_CLR_EOS:
      {
        var current_row = parm_form.field[parm_form.current].top;

        clear_eof (current_element);
        /* Forward till field on different row */
        for (i=parm_form.current + 1; i < real_form.length && current_row <= parm_form.field[i].top; i++)
        {
          if (is_field_input (real_form.elements[i]))
          {
            real_form.elements[i].value = "";
          }
          else if (real_form.elements[i].type == "checkbox")
          {
            real_form.elements[i].checked = false;
          }
        }
        real_form.elements[i-1].focus();
        break;
      }

      case REQ_BEG_FIELD:
      {
        begin_field (current_element);
        break;
      }

      case REQ_END_FIELD:
      {
        if (is_field_input (current_element))
        {
           if (current_element.size < current_element.maxLength)
           {
             for (i=current_element.value.length-1; i >= 0 && current_element.value.substr(i,1) == " "; i--);

             /* We have to first safely fire the delete beyond the actual
                data so increment i always */
             i++;
             set_cursor (current_element,i);

             /* Fire a delete at the end of the field to
                scroll overloaded field to right */
             fire_keycode (current_element, KEYCODE_DC, 1);

             /* Now backup if the field was full */
             if (i >= current_element.maxLength - 1) {
               set_cursor (current_element,--i);
             }
           }
           else
           {
             for (i=current_element.value.length-1; i >= 0 && current_element.value.substr(i,1) == " "; i--);
             if (i < current_element.maxLength - 1) {
               i++;
             }
             set_cursor (current_element, i);
           }
        }
        break;
      }

      case REQ_OVL_MODE:
      {
        NC_INSERT_TOGGLE = false;
        break;
      }

      case REQ_INS_MODE:
      {
        NC_INSERT_TOGGLE = true;
        break;
      }
    }
  }
  return (return_val);
}

function begin_field (parm_element)
{
  trace ("begin_field ("+parm_element.name+")");

  if (is_field_input (parm_element))
  {
    set_cursor (parm_element, 0);
    if (parm_element.size != parm_element.maxLength)
    {
      fire_keycode (parm_element,KEYCODE_BACKSPACE,1);           /* scroll overloaded field to left */
    }
  }
}

/*
*  Function: set_cursor ()
*
*  Description: Set the cursor to x,y
*
*/
function set_cursor (current_element,pos)
{
  trace ("set_cursor ("+current_element.name+", "+pos+")");

  if (is_field_input (current_element))
  {
    if (pos < 0) {
      pos = 0;
    }
    current_element.setSelectionRange(pos,pos);
  }
}

/*
*  Function: clear_eof ()
*
*  Description: Clear from cursor to end of a field
*
*/
function clear_eof (current_element)
{
  trace ("clear_eof ("+current_element.name+")");
  
  if (is_field_input (current_element))
  {
    current_element.value = current_element.value.substring(0, current_element.selectionStart);
  }
  else if (current_element.type == "checkbox")
  {
    current_element.checked = false;
  }
}


/*
* Description: Focus on first public field element in form
*
*/
function first_field (real_form)
{
  trace ("first_field ("+real_form.name+")");

  for (i=0; i < real_form.length && (real_form.elements[i].type == "hidden" || real_form.elements[i].disabled);i++);

  if (i < real_form.length)
  {
    real_form.elements[i].focus();

    /* Set Curser to beg of field */
    set_cursor (real_form.elements[i], 0);
    return true;
  }
  else {
    return false;
  }
}

/*
* Description: Focus on next public field in form
*
*/
function next_field (parm_form)
{
  var got_focus = false;
  var current_element = current_field(parm_form).element;
  var i;

  trace ("next_field ("+parm_form.name+")");
  
  for (i=current_element.tabIndex; i < parm_form.form.length; i++)
  {
    if (parm_form.form.elements[i].type != "hidden" && !parm_form.form.elements[i].disabled)
    {
      parm_form.form.elements[i].focus();
      got_focus = true;
      break;
    }
  }
  if (!got_focus)
  {       /* Set to the First element in form not hidden*/
    for (i=0; i < parm_form.form.length; i++)
    {
      if (parm_form.form.elements[i].type != "hidden" && !parm_form.form.elements[i].disabled)
      {
        parm_form.form.elements[i].focus();
        break;
      }
    }
  }
  set_cursor (parm_form.form.elements[i], 0);        /* Set Curser to beg of field */
}

/*
* Description: Focus on previous public field in form
*
*/
function prev_field (parm_form)
{
  var got_focus = false;
  var current_element = current_field(parm_form).element;
  var i;

  trace ("prev_field ("+parm_form.name+")");

  /* Tabindex is the next field, so minus 2 gives the previous field */
  for (i=current_element.tabIndex - 2; i >= 0; i--)
  {
    if (parm_form.form.elements[i].type != "hidden" && !parm_form.form.elements[i].disabled)
    {
      parm_form.form.elements[i].focus();
      got_focus = true;
      break;
    }
  }

  if (!got_focus)
  {       /* Set to the Last element in form not hidden*/
    for (i=parm_form.form.length - 1; i >= 0; i--)
    {
      if (parm_form.form.elements[i].type != "hidden" && !parm_form.form.elements[i].disabled)
      {
        parm_form.form.elements[i].focus();
        break;
      }
    }
   }
   /* Set Curser to beg of field */
   set_cursor (parm_form.form.elements[i], 0);
}


/*
* Function Set_field_buffer ()
*
* Description: Update the form->field object with the string
*/
function set_field_buffer (parm_field, parm_num, parm_buffer)
{
  trace ("set_field_buffer ("+parm_field.name+", "+parm_num+", "+parm_buffer+")");
  
  if (parm_field.element.tagName == "INPUT")
  {
    parm_field.buf = parm_buffer;
    parm_field.element.value = parm_buffer;
    parm_field.length = parm_buffer.length
  }
}


/*
* Function _nc_Refresh_Current_Field ()
*
* Description: Put the original buffer back
*/
function _nc_Refresh_Current_Field (parm_form)
{
  trace ("_nc_Refresh_Current_Field ("+parm_form.name+")");

  if (parm_form.field[parm_form.current].element.tagName == "INPUT")
  {
    parm_form.field[parm_form.current].element.value = parm_form.field[parm_form.current].buf;
  }
}


/*
* Function  _nc_Synchronize_Current_Field()
*
* Description: Update the internal buffer with current value
*/
function _nc_Synchronize_Current_Field (parm_form)
{
  trace ("_nc_Synchronize_Current_Field ("+parm_form.name+")");

  if (parm_form.field[parm_form.current].element.tagName == "INPUT")
  {
    parm_form.field[parm_form.current].buf = parm_form.field[parm_form.current].element.value;
    parm_form.field[parm_form.current].length = parm_form.field[parm_form.current].element.value.length;
  }
}


/*
* Function: Horizontal_Scrolling ()
*
* Description: Scroll off-field data into view
*
*/
function Horizontal_Scrolling (parm_form)
{
  var i,j,half_cols;
  var field,buffer;

  //trace ("Horizontal_Scrolling ("+parm_form.name+")");

  if (_HORZ_SCROLL_TIMER) {
    clearTimeout (_HORZ_SCROLL_TIMER);
    _HORZ_SCROLL_TIMER = 0;
  }

  if ((parm_form.status & _POSTED) && (parm_form.status & _HAS_HORZ_SCROLL))
  {
    for (i=0; i < parm_form.form.length; i++)
    {
      /* If they unpost the form on me */
      if (!(parm_form.status & _POSTED)) {
        return;
      }
      if (i != parm_form.current)
      {
        if (parm_form.field[i].element.tagName == "INPUT" && parm_form.field[i].cols < parm_form.field[i].maxgrow)
        {
          field = parm_form.field[i];
          buffer = field.buf;

          /* See if there is any off screen data */
          for (j=field.cols; j < field.length; j++)
          {
            if (buffer.substr(j,1) != " ") {
              break;
            }
          }

          if (j < field.length)
          {
            if (field.cur_col >= 0)
            {
              /* Prepare to scroll */
              half_cols = field.cols / 2;
              if (field.cur_col == 0) {
                field.element.value = field.buf;
                field.cur_col = half_cols;
              }

              /* Scroll */
              if (field.cur_col < field.maxgrow)
              {
                if (field.cur_col >= field.cols) {
                  field.element.value = field.element.value.substring(1);
                }
              }
              else
              {
                field.cur_col = -1;
                field.element.value = field.buf;
              }
              field.cur_col++;

              /* Control how far it scrolls off screen */
              for (j=(field.cur_col - half_cols); j >= 0 && j < field.length; j++)
              {
                if (buffer.substr(j,1) != " ") {
                  break;
                }
              }
              if (j >= field.length) {
                field.cur_col = -10;
              }
            }
            else
            {
              field.cur_col++;
            }
          }
        }
      }
    }
    _HORZ_SCROLL_TIMER = setTimeout("Horizontal_Scrolling(Main_form);", 200);
  }
}
