/*
*  JavaScript Implementation: jscurses
*
* Description:
*
*
* Author: Ricardo Cantu <ricky@smartcsc.com>, (C) 2006
*
* Copyright: See COPYING file that comes with this distribution
*
*/


/*
* Function: getch ()
*
* Description: Take a keycode and translate to curses
*              internal key.
*
*/

function getch (parm_event)
{
  var return_key = new _nc_Key_Type ();

  trace ("getch (event): keycode: "+parm_event.keyCode+" charCode: "+parm_event.charCode);

  return_key.charCode = parm_event.charCode;
  return_key.keyCode = parm_event.keyCode;
  
  if (parm_event.shiftKey) {
    return_key.key_mod = KEYCODE_SHIFT;
  }
  else if (parm_event.ctrlKey) {
    return_key.key_mod = KEYCODE_CTRL;
  }
  else if (parm_event.altKey) {
    return_key.key_mod = KEYCODE_ALT;
  }
  else {
    return_key.key_mod = 0;
  }

  /* Special keys will be in keycode and regular chars in charCode */
  if (parm_event.keyCode)
  {
    switch (parm_event.keyCode)
      {
          case KEYCODE_END:
          {
              return_key.key = KEY_END;
              break;
          }
          case KEYCODE_HOME:
          {
              return_key.key = KEY_HOME;
              break;
          }
          case KEYCODE_NPAGE:
          {
              return_key.key = KEY_NPAGE;
              break;
          }
          case KEYCODE_PPAGE:
          {
              return_key.key = KEY_PPAGE;
              break;
          }
          case KEYCODE_IC:
          {
              return_key.key = KEY_IC;
              break;
          }
          case KEYCODE_DC:
          {
              return_key.key = KEY_DC;
              break;
          }
          case KEYCODE_LEFT:
          {
              return_key.key = KEY_LEFT;
              break;
          }
          case KEYCODE_RIGHT:
          {
              return_key.key = KEY_RIGHT;
              break;
          }
          case KEYCODE_UP:
          {
              return_key.key = KEY_UP;
              break;
          }
          case KEYCODE_DOWN:
          {
              return_key.key = KEY_DOWN;
              break;
          }
          case KEYCODE_ESC:
          {
            return_key.key = KEY_ESC;
            break;
          }
          case KEYCODE_CAPSLOCK:
          {
            return_key.key = KEY_CAPSLOCK;
            break;
          }
          case KEYCODE_BACKSPACE:
          {
            return_key.key = KEY_BACKSPACE;
            break;
          }
          case KEYCODE_ENTER:
          {
            return_key.key = KEY_ENTER;
            break;
          }
          case KEYCODE_TAB:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_BTAB;
            }
            else {
              return_key.key = KEY_TAB;
            }
            break;
          }
          case KEYCODE_F1:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(13);
            }
            else {
              return_key.key = KEY_F(1);
            }
            break;
          }
          case KEYCODE_F2:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(14);
            }
            else {
              return_key.key = KEY_F(2);
            }
            break;
          }
          case KEYCODE_F3:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(15);
            }
            else {
              return_key.key = KEY_F(3);
            }
            break;
          }
          case KEYCODE_F4:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(16);
            }
            else {
              return_key.key = KEY_F(4);
            }
            break;
          }
          case KEYCODE_F5:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(17);
            }
            else {
              return_key.key = KEY_F(5);
            }
            break;
          }
          case KEYCODE_F6:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(18);
            }
            else {
              return_key.key = KEY_F(6);
            }
            break;
          }
          case KEYCODE_F7:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(19);
            }
            else {
              return_key.key = KEY_F(7);
            }
            break;
          }
          case KEYCODE_F8:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(20);
            }
            else {
              return_key.key = KEY_F(8);
            }
            break;
          }
          case KEYCODE_F9:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(21);
            }
            else {
              return_key.key = KEY_F(9);
            }
            break;
          }
          case KEYCODE_F10:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(22);
            }
            else {
              return_key.key = KEY_F(10);
            }
            break;
          }
          case KEYCODE_F11:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(23);
            }
            else {
              return_key.key = KEY_F(11);
            }
            break;
          }
          case KEYCODE_F12:
          {
            if (return_key.key_mod == KEYCODE_SHIFT) {
              return_key.key = KEY_F(24);
            }
            else {
              return_key.key = KEY_F(12);
            }
            break;
          }
      }
  }
  else
  {
      return_key.key = parm_event.charCode;
  }

  return return_key;
}
